module AdviceApplicator where

import Control.Monad(foldM)
import Data.Char(isAlphaNum)

import Input
import Parser
import Result
import Utils(split)

-- | Takes an operation declaration, an equation declaration and an advice.
-- Returns a list of pairs representing the mapping.
mapVariables :: String -> String -> AdviceSen -> Result [(String, String)]
mapVariables op eq ad = do
  eqVars <- findVariables op eq
  zipVars adVars eqVars
  where
    adVars = adviceInputs ad
    zipVars :: [String] -> [String] -> Result [(String, String)]
    zipVars vars1 vars2
      | length vars1 /= length vars2 = Error "Could not map variables."
      | otherwise                    = Value (zip vars1 vars2)

-- | Takes an operation declaration (from operation context) and an equation
-- declaration. Find input variables in the equation.
findVariables :: String -> String -> Result [String]
findVariables op eq
  | length opDTokens /= length eqDTokens = Error ("Syntax error near " ++ eq)
  | otherwise                            = foldResult join $ filter (uncurry (/=)) zipped
  where
    opDTokens = words op
    eqDTokens = words eq
    zipped = zip opDTokens eqDTokens
    join :: (String, String) -> Result String
    join (o, e)
      | o == "_" = Value e
      | o /= e = Error ("Operation `" ++ op ++ "` does not match equation `" ++ eq ++ "`")

-- | Takes a mapping and a body of an advice.
-- Returns the body with applied mapping.
applyMapping :: [(String, String)] -> String -> Result String
applyMapping m = Value . unwords . map (remap m) . swords
  where
    remap :: [(String, String)] -> String -> String
    remap [] t                          = t
    remap ((a, o) : rest) t | a == t    = o
                            | otherwise = remap rest t

-- | Takes a whole equation (not just declaration) and an advice body.
-- Returns the equation with the advice body included inside on the right spots.
rewriteEquation :: String -> String -> String
rewriteEquation eq ad =
  eqHead ++ " " ++ unwords (map (replace adVar adGoal) $ words eqBody)
  where
    eqHead = (unwords . takeWhile (/= "=") . words) eq
    eqBody = (unwords . dropWhile (/= "=") . words) eq
    adWrds = words ad
    adVar  = head adWrds
    adGoal = unwords $ drop 2 adWrds
    replace :: String -> String -> String -> String
    replace var goal x
      | var == x  = goal
      | otherwise = x

-- | Takes an operation declaration (from opcontext), a whole equation
-- and an advice. Returns the equation with the advice applied.
applyAdvice :: String -> String -> AdviceSen -> Result String
applyAdvice op eq ad = do
  mapping  <- mapVariables op eqDecl ad
  mappedAd <- applyMapping mapping adBody
  return (rewriteEquation eq mappedAd)
  where
    adBody = adviceBody ad
    eqDecl = eqDeclaration eq

-- | Takes an operation declaration (from opcontext), a whole equation
-- and a list of advices. Returns the equation with all the advices applied (in given order).
applyAdvices :: String -> String -> [AdviceSen] -> Result String
applyAdvices op = foldM (applyAdvice op)

-- | Sensitive words - uses any non-alphanum char as delimiter
swords :: String -> [String]
swords = concatMap (split (not . isAlphaNum)) . words
