module CommandLineInterface where

import Result

data Settings = Settings { inputFile  :: String,
                           outputFile :: String,
                           showHelp   :: Bool} deriving (Eq, Show)

type ArgsFun = (Settings -> [String] -> (Result Settings, Int))


-- | Takes a list of arguments and returns settings.
cli :: [String] -> Result Settings
cli = setup defaultSettings helpFun outputFun inputFun

-- | Takes a settings and changes it according to the
-- input string.
setup :: Settings -> ArgsFun -> ArgsFun -> ArgsFun -> [String] -> Result Settings
setup settings _ _ _ [] = Error "Invalid arguments."
setup settings helpF outputF inputF is
  | i == "-h"  =
    case helpF settings is of
      (Error e, _)      -> Error e
      (Value set, _)    -> Value set
  | i == "-o"  =
    case outputF settings is of
      (Error e, _)      -> Error e
      (Value set, skip) -> setup set helpF outputF inputF (drop skip is)
  | isSwitch i = Error ("Unexpected flag " ++ i)
  | otherwise  =
    case inputF settings is of
      (Error e, _)      -> Error e
      (Value set, _)    -> Value set
  where
    i           = head is

-- | Returns default settings for the application
defaultSettings :: Settings
defaultSettings = Settings { inputFile  = "",
                             outputFile = "a.maude",
                             showHelp   = False}

helpFun :: ArgsFun
helpFun s _ = (Value Settings { inputFile = inputFile s, outputFile = outputFile s, showHelp = True }, 1)

outputFun :: ArgsFun
outputFun _ [] = (Error "Invalid arguments.", 0)
outputFun s is
  | null afterFlag   = (Error "Output file flag was set but the file was not specified.", 0)
  | isSwitch outFile = (Error ("Unexpected flag " ++ outFile), 0)
  | otherwise        = (Value Settings {inputFile = inputFile s, outputFile = outFile, showHelp = showHelp s}, 2)
  where
    afterFlag = tail is
    outFile   = head afterFlag

inputFun :: ArgsFun
inputFun _ [] = (Error "Missing input.", 0)
inputFun s is
  | length is > 1 = (Error "Too many arguments.", 0)
  | otherwise     = (Value Settings {inputFile = head is, outputFile = outputFile s, showHelp = showHelp s}, 1)

-- | Returns true if the given string is a switch,
-- i.e. starts with `-` such as `-h` or `-o`.
isSwitch :: String -> Bool
isSwitch ('-':_) = True
isSwitch _       = False