module Parser where

import Result
import Utils(isAlphaNum)

data AspectSen = AspectSen String Int String deriving (Eq, Show)
data AdviceSen = AdviceSen String [String] String deriving (Eq, Show)

-- | Gets name from an aspect sentence
aspectName :: AspectSen -> String
aspectName (AspectSen name _ _) = name

-- | Gets number of inputs from an aspect sentence
aspectInputs :: AspectSen -> Int
aspectInputs (AspectSen _ inputs _) = inputs

-- | Gets pointcut from an aspect sentence
aspectPointcut :: AspectSen -> String
aspectPointcut (AspectSen _ _ pointcut) = pointcut

-- | Gets aspect name from an advice sentence
adviceAspect :: AdviceSen -> String
adviceAspect (AdviceSen asp _ _) = asp

-- | Gets list of inputs from an advice sentence
adviceInputs :: AdviceSen -> [String]
adviceInputs (AdviceSen _ inputs _) = inputs

-- | Gets advice body from an advice sentence
adviceBody :: AdviceSen -> String
adviceBody (AdviceSen _ _ body) = body

-- | Takes a String containing an aspect sentence.
-- Performs a validation a creates AspectSen.
parseAspect :: String -> Result AspectSen
parseAspect s
  | length wrds < 7       = Error ("Syntax error near " ++ s)
  | keyword /= "as"       = Error ("Wrong keyword in " ++ s)
  | last wrds /= "."      = Error ("Missing dot in " ++ s)
  | not $ isAlphaNum name = Error ("The aspect name may contain only alphanum chars. Near " ++ s)
  | ":" `notElem` wrds    = Error ("Missing a colon near " ++ s)
  | notInputs /= []       = Error ("Syntax error in the inputs part near " ++ s)
  | inputsLen < 1         = Error ("Aspect must have at least a single input near " ++ s)
  | inputsLen /= pclen    = Error ("Number of inputs in the aspect and the pointcut must be equal near " ++ s)
  | otherwise             = Value (AspectSen name inputsLen (unwords pc))
  where
    wrds                  = words s
    keyword               = head wrds
    name                  = wrds !! 1 --second word in the s (after keyword)
    inputsPart            = takeWhile (/= ":") $ drop 2 wrds
    inputs                = filter (== "_") inputsPart
    inputsLen             = length inputs
    notInputs             = filter (/= "_") inputsPart --words present in the inputsPart which are not inputs
    pc                    = takeWhile (/= ".") $ drop 1 $ dropWhile (/= ":") wrds --words between `:` and `.`
    pclen                 = length $ filter (== "_") pc

-- | Takes a String containing an advice sentence.
-- Performs a validation a creates AspectSen.
parseAdvice :: String -> Result AdviceSen
parseAdvice s
  | length wrds < 8         = Error ("Syntax error near " ++ s)
  | keyword /= "ad"         = Error ("Wrong keyword in " ++ s)
  | last wrds /= "."        = Error ("Missing dot in " ++ s)
  | not $ isAlphaNum name   = Error ("The aspect name may contain only alphanum chars. Near " ++ s)
  | nAlphaNumInputs /= []   = Error ("Names of variables must be alphanumeric near " ++ s)
  | length body < 3         = Error ("Syntax error near " ++ s)
  | eqSign /= "="           = Error ("Missing `=` in " ++ s)
  | varSet `notElem` inputs = Error ("The variable being rewritten must be declared in the advice. Near " ++ s)
  | otherwise               = Value (AdviceSen name inputs (unwords body))
  where
    wrds                  = words s
    keyword               = head wrds
    name                  = wrds !! 1
    inputs                = takeWhile (/= ":") $ drop 2 wrds
    nAlphaNumInputs       = filter (not . isAlphaNum) inputs
    body                  = takeWhile (/= ".") $ drop 1 $ dropWhile (/= ":") wrds
    varSet                = head body
    eqSign                = body !! 1

-- | Takes a list of lines and returns AspectSens created from it.
parseAspects :: [String] -> Result [AspectSen]
parseAspects = foldResult parseAspect

-- | Takes a list of lines and returns AdviceSens created from it.
parseAdvices :: [String] -> Result [AdviceSen]
parseAdvices = foldResult parseAdvice

-- | Returns the result of the given function when
-- applied on the keyword from the given string.
isStmtOfType :: String -> String -> Bool
isStmtOfType _ ""   = False
isStmtOfType kw s   = ((== kw) . head . words) s

-- | Returns true if the given sentence is of type `as`
isAspect :: String -> Bool
isAspect = isStmtOfType "as"

-- | Returns true if the given sentence is of type `ad`
isAdvice :: String -> Bool
isAdvice = isStmtOfType "ad"

-- | Returns true if the given sentence is of type `op`
isOperation :: String -> Bool
isOperation = isStmtOfType "op"

-- | Returns true if the given sentence is of type `eq`
isEquation :: String -> Bool
isEquation = isStmtOfType "eq"

-- | Parses declaration depending on the given delimiter.
-- Assumes that the sentence keyword is on the first place in
-- the sentence.
parseDeclaration :: String -> String -> String
parseDeclaration delim = unwords . drop 1 . takeWhile (/= delim) . words

-- | Parses declaration of an operation.
-- `op foo _ _ : Nat Nat -> Nat .` => `foo _ _`
opDeclaration :: String -> String
opDeclaration = parseDeclaration ":"

-- | Parses declaration of an equation.
-- `eq foo a b = a + b .` => `foo a b`
eqDeclaration :: String -> String
eqDeclaration = parseDeclaration "="
