module Result where

-- | Result data type represents a result of a function which
-- can end either successfully or unsuccessfully.
data Result a = Value a | Error String

instance (Eq a) => Eq (Result a) where
  (Value x) == (Value y) = x == y
  (Value _) == (Error _) = False
  (Error _) == (Value _) = False
  (Error _) == (Error _) = True

instance (Show a) => Show (Result a) where
  show (Value x) = show x
  show (Error x) = show x

instance Functor Result where
  fmap f (Value x) = Value (f x)
  fmap _ (Error x) = Error x

instance Applicative Result where
  pure = Value
  (Error x) <*> _ = Error x
  (Value f) <*> x = fmap f x

instance Monad Result where
  return = pure
  Error x >>= f = Error x
  Value x >>= f = f x
  fail = Error

-- | Returns true if the given input is an error.
isError :: Result a -> Bool
isError (Error _) = True
isError _         = False

-- | Returns true if the given input is a value.
isValue :: Result a -> Bool
isValue = not . isError

-- | Gets a value from Value
fromValue :: Result a -> a
fromValue (Value v) = v
fromValue (Error _) = error "could not get value from error"

-- | Applies the given function to each element of the given list.
-- If the function results in an Error when applied on a particular element
-- result of the folding is the Error. Otherwise the result is list of single
-- results wrapped in Value.
foldResult :: (a -> Result b) -> [a] -> Result [b]
foldResult _ []     = Value []
foldResult f (l:ls) = do
  v  <- f l
  v' <- foldResult f ls
  return (v:v')