module ParserTest where

import Test.HUnit

import Parser
import Result

tests = TestList[
    TestLabel "ParseAspectTest" testParseAspect,
    TestLabel "ParseAdviceTest" testParseAdvice,
    TestLabel "ParseDeclarationTest" testParseDeclaration,
    TestLabel "OpDeclarationTest" testOpDeclaration,
    TestLabel "EqDeclarationTest" testEqDeclaration,
    TestLabel "IsAspectTest" testIsAspect,
    TestLabel "IsAdviceTest" testIsAdvice,
    TestLabel "IsOperationTest" testIsOperation,
    TestLabel "IsEquationTest" testIsEquation,
    TestLabel "AspectFunctionsTest" testAspectFunctions,
    TestLabel "AdviceFunctionsTest" testAdviceFunctions,
    TestLabel "ParseAspectsTest" testParseAspects,
    TestLabel "ParseAdvicesTest" testParseAdvices]

testParseAspect = TestCase (do
  assertEqual "checking valid aspect" (Value (AspectSen "foo" 1 "bar _")) $ parseAspect "as foo _ : bar _ ."
  assertEqual "checking empty str" (Error "") $ parseAspect ""
  assertEqual "checking invalid aspect" (Error "") $ parseAspect "as foo _ : bar _ _ ."
  assertEqual "checking invalid aspect" (Error "") $ parseAspect "as foo : bar _ ."
  assertEqual "checking invalid aspect" (Error "") $ parseAspect "as fo-o _ : bar _ ."
  assertEqual "checking invalid aspect" (Error "") $ parseAspect "as foo _ _: bar _ ."
  assertEqual "checking invalid aspect" (Error "") $ parseAspect "as foo __ : bar _ ."
  assertEqual "checking invalid aspect" (Error "") $ parseAspect "as foo_ : bar _ ."
  assertEqual "checking invalid aspect" (Error "") $ parseAspect "asfoo _ : bar _ ."
  assertEqual "checking invalid aspect" (Error "") $ parseAspect "ad foo _ : bar _ ."
  assertEqual "checking invalid aspect" (Error "") $ parseAspect "foo _ : bar _ ."
  assertEqual "checking invalid aspect" (Error "") $ parseAspect "as foo _ : bar _ bar " --missing dot
  assertEqual "checking invalid aspect" (Error "") $ parseAspect "as foo _ bar _ ." --missing colon
  assertEqual "checking invalid aspect" (Error "") $ parseAspect "as foo : _ bar _ .")

testParseAdvice = TestCase (do
  assertEqual "checking valid advice" (Value (AdviceSen "foo" ["a", "b"] "a = b + 2")) $ parseAdvice "ad foo a b : a = b + 2 ."
  assertEqual "checking empty str" (Error "") $ parseAdvice ""
  assertEqual "checking invalid advice" (Error "") $ parseAdvice "ad foo a b : a = b + 2"
  assertEqual "checking invalid advice" (Error "") $ parseAdvice "ad foo a b : x = b + 2 ."
  assertEqual "checking invalid advice" (Error "") $ parseAdvice "as foo a b : a = b + 2 ."
  assertEqual "checking invalid advice" (Error "") $ parseAdvice "ad fo_o a b : a = b + 2 ."
  assertEqual "checking invalid advice" (Error "") $ parseAdvice "ad foo a _ b : a = b + 2 ."
  assertEqual "checking invalid advice" (Error "") $ parseAdvice "ad foo _ a b : a = b + 2 ."
  assertEqual "checking invalid advice" (Error "") $ parseAdvice "ad foo a b a = b + 2 ."
  assertEqual "checking invalid advice" (Error "") $ parseAdvice "ad foo a b : a = ."
  assertEqual "checking invalid advice" (Error "") $ parseAdvice "ad foo a b c d e f : a b c d e .")


testParseDeclaration = TestCase(do
  assertEqual "Testing declaration parsing" "foo _ _" $ parseDeclaration ":" "op foo _ _ : Nat Nat -> Nat ."
  assertEqual "Testing declaration parsing" "foo a b" $ parseDeclaration "=" "eq foo a b = a plus b .")

testOpDeclaration = TestCase(do
  assertEqual "Testing operation declaration parsing" "foo _ _" $ opDeclaration "op foo _ _ : Nat Nat -> Nat ."
  assertEqual "Testing operation declaration parsing" "_ foo _" $ opDeclaration "op _ foo _ : Nat Nat -> Nat .")

testEqDeclaration = TestCase(do
  assertEqual "Testing equation declaration parsing" "foo a b" $ eqDeclaration "eq foo a b = a + b ."
  assertEqual "Testing equation declaration parsing" "a foo b" $ eqDeclaration "eq a foo b = a + b .")

testIsAspect = TestCase(do
  assertBool "Testing isAspect" $ isAspect "as foo _ _ : _ bar _ ."
  assertBool "Testing isAspect" $ not $ isAspect "ad foo a b : a = b .")

testIsAdvice = TestCase(do
  assertBool "Testing isAdvice" $ isAdvice "ad foo a b : a = b ."
  assertBool "Testing isAdvice" $ not $ isAdvice ""
  assertBool "Testing isAdvice" $ not $ isAdvice "as foo _ _ : _ bar _ .")

testIsOperation = TestCase(do
  assertBool "Testing isOperation" $ isOperation "op foo _ _ : _ bar _ ."
  assertBool "Testing isOperation" $ not $ isOperation "ad foo a b : a = b .")

testIsEquation = TestCase(do
  assertBool "Testing isOperation" $ isEquation "eq foo a b = a + b ."
  assertBool "Testing isOperation" $ not $ isEquation "ad foo a b : a = b .")

testAspectFunctions = TestCase(do
  assertEqual "testing aspectName" "name" $ aspectName (AspectSen "name" 2 "pointcut")
  assertEqual "testing aspectInputs" 2 $ aspectInputs (AspectSen "name" 2 "pointcut")
  assertEqual "testing aspectPointcut" "pointcut" $ aspectPointcut (AspectSen "name" 2 "pointcut"))

testAdviceFunctions = TestCase(do
  assertEqual "testing adviceName" "name" $ adviceAspect (AdviceSen "name" ["a", "b"] "body")
  assertEqual "testing adviceInputs" ["a", "b"] $ adviceInputs (AdviceSen "name" ["a", "b"] "body")
  assertEqual "testing adviceBody" "body" $ adviceBody (AdviceSen "name" ["a", "b"] "body"))

testParseAspects = TestCase(
  assertEqual "testing parseAspects" (Value [AspectSen "foo" 2 "pointcut _ _", AspectSen "foo2" 1 "pointcut2 _"]) $ parseAspects ["as foo _ _ : pointcut _ _ .", "as foo2 _ : pointcut2 _ ."])

testParseAdvices = TestCase(
  assertEqual "testing parseAdvices" (Value [AdviceSen "foo" ["a", "b"] "a = b", AdviceSen "foo2" ["x"] "x = 0"]) $ parseAdvices ["ad foo a b : a = b .", "ad foo2 x : x = 0 ."])